import 'package:comunic/helpers/account_helper.dart';
import 'package:comunic/ui/widgets/dialogs/cancel_dialog_button.dart';
import 'package:comunic/ui/widgets/safe_state.dart';
import 'package:comunic/utils/input_utils.dart';
import 'package:comunic/utils/intl_utils.dart';
import 'package:comunic/utils/ui_utils.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

/// Reset password route
///
/// @author Pierre Hubert

class ResetPasswordRoute extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text(tr("Password forgotten")),
      ),
      body: Center(
        child: ConstrainedBox(
          constraints: BoxConstraints(maxWidth: 300),
          child: _ResetPasswordBody(),
        ),
      ),
    );
  }
}

enum _SelectedOption { NONE }

class _ResetPasswordBody extends StatefulWidget {
  @override
  _ResetPasswordBodyState createState() => _ResetPasswordBodyState();
}

class _ResetPasswordBodyState extends SafeState<_ResetPasswordBody> {
  var _loading = false;

  /// Step 1 - check email address
  String _emailAddress;
  final _emailController = TextEditingController();

  String get _inputEmail => _emailController.text;

  bool get _isEmailValid =>
      _inputEmail.isNotEmpty && validateEmail(_inputEmail);

  /// Step 2 - Offer options
  bool _hasSecurityQuestions;

  var _selectedOption = _SelectedOption.NONE;

  void _setLoading(bool loading) => setState(() => _loading = loading);

  @override
  Widget build(BuildContext context) {
    if (_loading) return buildCenteredProgressBar();

    if (_emailAddress == null) return _buildEnterEmailAddressScreen();

    switch (_selectedOption) {
      case _SelectedOption.NONE:
        return _buildOptionsScreen();
    }
  }

  Widget _buildEnterEmailAddressScreen() {
    return Column(
      mainAxisSize: MainAxisSize.min,
      children: <Widget>[
        Text(tr("Please enter your email address to reset your password:")),
        TextField(
          controller: _emailController,
          onChanged: (s) => setState(() {}),
          onSubmitted: _isEmailValid ? (s) => _checkEmail() : null,
          textInputAction: TextInputAction.done,
          keyboardType: TextInputType.emailAddress,
          decoration: InputDecoration(
            icon: Icon(Icons.email),
            alignLabelWithHint: true,
            labelText: tr("Email address..."),
            suffixIcon: IconButton(
              icon: Icon(Icons.check),
              onPressed: _isEmailValid ? _checkEmail : null,
            ),
            errorText: _inputEmail.isEmpty || _isEmailValid
                ? null
                : tr("Invalid email address!"),
          ),
        ),
      ],
    );
  }

  /// Check given email address
  void _checkEmail() async {
    try {
      _setLoading(true);

      // Check if email address exists or not
      if (!await AccountHelper.existsMailAccount(_inputEmail)) {
        _setLoading(false);
        showSimpleSnack(context, tr("Specified email address was not found!"));
        return;
      }

      _hasSecurityQuestions =
          await AccountHelper.hasSecurityQuestions(_inputEmail);

      // We retain email address only if everything went well
      _emailAddress = _inputEmail;
      _setLoading(false);
    } catch (e, s) {
      print("Could not check given email! $e\n$s");
      showSimpleSnack(
          context, tr("An error occurred while checking your options !"));
      _setLoading(false);
    }
  }

  /// Offer the user the options he has to reset his account
  Widget _buildOptionsScreen() => Column(
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.stretch,
        children: <Widget>[
          Text(tr("Here are your options to reset your account:")),
          Container(height: 15),
          OutlineButton.icon(
            onPressed: _openSendEmailDialog,
            icon: Icon(Icons.email),
            label: Text(tr("Send us an email to ask for help")),
          ),
          Container(height: 15),
        ],
      );

  /// Show a dialog with our email address
  void _openSendEmailDialog() {
    showDialog(
        context: context,
        builder: (c) => AlertDialog(
              title: Text("Contact us"),
              content: Text(tr("You can reach us at contact@communiquons.org")),
              actions: <Widget>[CancelDialogButton()],
            ));
  }
}
